@@ -1,5 +1,5 @@
 /* source: xio-progcall.c */
-/* Copyright Gerhard Rieger 2001-2009 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains common code dealing with program calls (exec, system) */
@@ -229,17 +229,17 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	       Warn2("unlockpt(%d): %s", ptyfd, strerror(errno));
 	    }
 #endif /* HAVE_UNLOCKPT */
-#if HAVE_PTSNAME	/* AIX, not Linux */
+#if HAVE_PROTOTYPE_LIB_ptsname	/* AIX, not Linux */
 	    if ((tn = Ptsname(ptyfd)) == NULL) {
 	       Warn2("ptsname(%d): %s", ptyfd, strerror(errno));
 	    }
-#endif /* HAVE_PTSNAME */
+#endif /* HAVE_PROTOTYPE_LIB_ptsname */
 	    if (tn == NULL) {
 	       if ((tn = Ttyname(ptyfd)) == NULL) {
-		  Warn2("ttyname(%d): %s", ptyfd, strerror(errno));
+		  Error2("ttyname(%d): %s", ptyfd, strerror(errno));
 	       }
 	    }
-	    strncpy(ptyname, tn, MAXPTYNAMELEN);
+	    ptyname[0] = '\0'; strncat(ptyname, tn, MAXPTYNAMELEN-1);
 	    if ((ttyfd = Open(tn, O_RDWR|O_NOCTTY, 0620)) < 0) {
 	       Warn2("open(\"%s\", O_RDWR|O_NOCTTY, 0620): %s", tn, strerror(errno));
 	    } else {
@@ -414,8 +414,10 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       gid_t group;
 
       if (withfork) {
-	 if (Signal(SIGCHLD, SIG_IGN) == SIG_ERR) {
-	    Warn1("signal(SIGCHLD, SIG_IGN): %s", strerror(errno));
+	 /* The child should have default handling for SIGCHLD. */
+	 /* In particular, it's not defined whether ignoring SIGCHLD is inheritable. */
+	 if (Signal(SIGCHLD, SIG_DFL) == SIG_ERR) {
+	    Warn1("signal(SIGCHLD, SIG_DFL): %s", strerror(errno));
 	 }
 
 #if HAVE_PTY
