@@ -1,5 +1,5 @@
 /* source: xio-socks.c */
-/* Copyright Gerhard Rieger 2001-2011 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of socks4 type */
@@ -236,7 +236,7 @@ int _xioopen_socks4_prepare(const char *targetport, struct opt *opts, char **soc
 	 }
       }
    }
-   strncpy(sockhead->userid, userid, *headlen-SIZEOF_STRUCT_SOCKS4);
+   sockhead->userid[0] = '\0'; strncat(sockhead->userid, userid, *headlen-SIZEOF_STRUCT_SOCKS4-1);
    *headlen = SIZEOF_STRUCT_SOCKS4+strlen(userid)+1;
    return STAT_OK;
 }
@@ -279,7 +279,7 @@ int
          after the user name's trailing 0 byte.  */
       char* insert_position = (char*) sockhead + *headlen;
 
-      strncpy(insert_position, hostname, BUFF_LEN-*headlen);
+      insert_position[0] = '\0'; strncat(insert_position, hostname, BUFF_LEN-*headlen-1);
       ((char *)sockhead)[BUFF_LEN-1] = 0;
       *headlen += strlen(hostname) + 1;
       if (*headlen > BUFF_LEN) {
@@ -364,7 +364,7 @@ int _xioopen_socks4_connect(struct single *xfd,
 	 char msgbuff[3*SIZEOF_STRUCT_SOCKS4];
 	 * xiohexdump((const unsigned char *)replyhead+bytes, result, msgbuff)
 	    = '\0';
-	 Debug2("received socks4 reply data (offset %u): %s", bytes, msgbuff);
+	 Debug2("received socks4 reply data (offset "F_Zd"): %s", bytes, msgbuff);
       }
 #endif /* WITH_MSGLEVEL <= E_DEBUG */
       bytes += result;
@@ -372,7 +372,7 @@ int _xioopen_socks4_connect(struct single *xfd,
 	 Debug1("received all "F_Zd" bytes", bytes);
 	 break;
       }
-      Debug2("received "F_Zd" bytes, waiting for "F_Zu" more bytes",
+      Debug2("received %d bytes, waiting for "F_Zu" more bytes",
 	     result, SIZEOF_STRUCT_SOCKS4-bytes);
    }
    if (result <= 0) {	/* we had a problem while reading socks answer */
