@@ -1,5 +1,5 @@
 /* source: xioshutdown.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended shutdown function */
@@ -31,7 +31,36 @@ int xioshutdown(xiofile_t *sock, int how) {
       if ((how+1)&2) {
 	 result |= xioshutdown((xiofile_t *)sock->dual.stream[1], 1);
       }
+      return result;
+   }
 
+   switch (sock->stream.howtoshut) {
+      char writenull;
+   case XIOSHUT_NONE:
+      return 0;
+   case XIOSHUT_CLOSE:
+      if (Close(sock->stream.fd) < 0) {
+	 Info2("close(%d): %s",
+	       sock->stream.fd, strerror(errno));
+      }
+      return 0;
+   case XIOSHUT_DOWN:
+      if ((result = Shutdown(sock->stream.fd, how)) < 0) {
+	 Info3("shutdown(%d, %d): %s",
+	       sock->stream.fd, how, strerror(errno));
+      }
+      return 0;
+#if _WITH_SOCKET
+   case XIOSHUT_NULL:
+      /* send an empty packet; only useful on datagram sockets? */
+      xiowrite(sock, &writenull, 0);
+      return 0;
+#endif /* _WITH_SOCKET */
+   default: ;
+   }
+
+   if (false) {
+      ;
 #if WITH_OPENSSL
    } else if ((sock->stream.dtype & XIODATA_MASK) == XIODATA_OPENSSL) {
       sycSSL_shutdown (sock->stream.para.openssl.ssl);
@@ -98,7 +127,8 @@ int xioshutdown(xiofile_t *sock, int how) {
 	 }
 	 Alarm(0);
       }
-   } else if ((sock->stream.dtype & XIODATA_MASK) == XIODATA_RECVFROM) {
+   } else if ((sock->stream.dtype & XIODATA_MASK) ==
+	      (XIODATA_RECVFROM & XIODATA_MASK)) {
       if (how >= 1) {
 	 if (Close(sock->stream.fd) < 0) {
 	    Info2("close(%d): %s",
